/*------------------------ BEGIN small_blas.h BEGIN -----------------------*/
/* @file small_blas.h                                                      */
/* @author jlinford                                                        */
/* @date 2012-06-22 15:44:03.915248                                        */
/* @brief Basic linear algebra subprogram definitions                      */
/*                                                                         */
/* A reduced set of BLAS routines optimized for Kppa-generated solvers     */
/*                                                                         */
/* This file was generated by Kppa: http://www.paratools.com/Kppa          */
/*-------------------------------------------------------------------------*/

#ifndef __SMALL_BLAS_H__
#define __SMALL_BLAS_H__

#ifdef __cplusplus
extern "C" {
#endif


/*--------------------------------- WCOPY ---------------------------------*/
/* Copies vector x to vector y: y <= x                                     */
/* Like the BLAS {S,D}COPY(N,X,1,Y,1)                                      */
/*                                                                         */
/* @param[in]     ncells32 A multiple of 32 grid cells                     */
/* @param[in]     n        Vector length                                   */
/* @param[in]     d_x      Vector x in device memory                       */
/* @param[out]    d_y      Vector y in device memory                       */
/*-------------------------------------------------------------------------*/
void WCOPY(unsigned long const ncells32, unsigned long const n, float d_x[],
        float d_y[]);


/*--------------------------------- WSCAL ---------------------------------*/
/* Constant times a vector: x <= alpha*x                                   */
/* Like the BLAS {S,D}SCAL(N,alpha,X,1)                                    */
/*                                                                         */
/* @param[in]     ncells32 A multiple of 32 grid cells                     */
/* @param[in]     n        Vector length                                   */
/* @param[in]     alpha    Scalar                                          */
/* @param[in,out] d_x      Vector x in device memory                       */
/*-------------------------------------------------------------------------*/
void WSCAL(unsigned long const ncells32, unsigned long const n, float const
        alpha, float d_x[]);


/*--------------------------------- WAXPY ---------------------------------*/
/* Constant times a vector plus a vector: y <= y + alpha*x                 */
/* Like the BLAS {S,D}AXPY(N,alpha,X,1,Y,1)                                */
/*                                                                         */
/* @param[in]     ncells32 A multiple of 32 grid cells                     */
/* @param[in]     n        Vector length                                   */
/* @param[in]     alpha    Scalar                                          */
/* @param[in]     d_x      Vector x in device memory                       */
/* @param[in,out] d_y      Vector y in device memory                       */
/*-------------------------------------------------------------------------*/
void WAXPY(unsigned long const ncells32, unsigned long const n, float const
        alpha, float d_x[], float d_y[]);


/*--------------------------------- WYMXDA --------------------------------*/
/* Difference of two vectors divided by a constant: z <= (y - x) / alpha   */
/*                                                                         */
/* @param[in]     ncells32 A multiple of 32 grid cells                     */
/* @param[in]     n        Vector length                                   */
/* @param[in]     d_x      Vector x in device memory                       */
/* @param[in]     d_y      Vector y in device memory                       */
/* @param[in]     alpha    Scalar                                          */
/* @param[out]    d_z      Vector z in device memory                       */
/*-------------------------------------------------------------------------*/
void WYMXDA(unsigned long const ncells32, unsigned long const n, float d_x[],
        float d_y[], float const alpha, float d_z[]);



#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* __SMALL_BLAS_H__ */
/*-------------------------- END small_blas.h END -------------------------*/
