/*------------------- BEGIN small_cu_parameters.h BEGIN -------------------*/
/* @file small_cu_parameters.h                                             */
/* @author jlinford                                                        */
/* @date 2012-06-22 15:44:03.795103                                        */
/* @brief CUDA interface to program parameters                             */
/*                                                                         */
/* CUDA interface to program parameters, such as integration tolerances,   */
/* program constants, and species indices.                                 */
/*                                                                         */
/* This file was generated by Kppa: http://www.paratools.com/Kppa          */
/*-------------------------------------------------------------------------*/

#ifndef __SMALL_CU_PARAMETERS_H__
#define __SMALL_CU_PARAMETERS_H__

#ifdef __cplusplus
extern "C" {
#endif


/*-------------------------------------------------------------------------*/
/* Integration tolerances                                                  */
/*-------------------------------------------------------------------------*/

/* Absolute tolerance */
#define ATOLS            (float)(1.0)
/* Relative tolerance */
#define RTOLS            (float)(0.001)


/*-------------------------------------------------------------------------*/
/* Concentration constants                                                 */
/*-------------------------------------------------------------------------*/

/* Conversion factor */
#define CFACTOR          (float)(1.0)
/* Default initialization value for variable species */
#define VAR_DEFAULT      (float)(0.0)
/* Default initialization value for fixed species */
#define FIX_DEFAULT      (float)(0.0)


/*-------------------------------------------------------------------------*/
/* Program constants                                                       */
/*-------------------------------------------------------------------------*/

/* Species count */
#define NSPEC            (unsigned long)(7)
/* Variable species count */
#define NVAR             (unsigned long)(5)
/* Fixed species count */
#define NFIX             (unsigned long)(2)
/* Active variable species count */
#define NVARACT          (unsigned long)(5)
/* Reaction (equation) count */
#define NREACT           (unsigned long)(10)
/* Number of nonzero entries in the Jacobian */
#define NONZERO          (unsigned long)(19)
/* Number of nonzero entries in the LU decomposition of the Jacobian */
#define LU_NONZERO       (unsigned long)(19)


/*-------------------------------------------------------------------------*/
/* Numerical constants                                                     */
/*-------------------------------------------------------------------------*/

#define ZERO             (float)(0.0)
#define HALF             (float)(0.5)
#define ONE              (float)(1.0)
#define TWO              (float)(2.0)


/*-------------------------------------------------------------------------*/
/* Variable species indices                                                */
/*-------------------------------------------------------------------------*/

#define IND_O1D          (unsigned long)(0)
#define IND_O            (unsigned long)(1)
#define IND_O3           (unsigned long)(2)
#define IND_NO           (unsigned long)(3)
#define IND_NO2          (unsigned long)(4)


/*-------------------------------------------------------------------------*/
/* Fixed species indices                                                   */
/*-------------------------------------------------------------------------*/

#define IND_M            (unsigned long)(5)
#define IND_O2           (unsigned long)(6)



#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* __SMALL_CU_PARAMETERS_H__ */
/*--------------------- END small_cu_parameters.h END ---------------------*/
