/*---------------------- BEGIN small_decomp.cu BEGIN ----------------------*/
/* @file small_decomp.cu                                                   */
/* @author jlinford                                                        */
/* @date 2012-06-22 15:44:04.069143                                        */
/* @brief LU decomposition of the row-compressed sparse Jacobian           */
/*                                                                         */
/* LU decomposition of the row-compressed sparse Jacobian                  */
/*                                                                         */
/* This file was generated by Kppa: http://www.paratools.com/Kppa          */
/*-------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <math.h>
#include "small_cu_parameters.h"
#include "small_sparse.h"
#include "small_decomp.h"

/*-------------------------------- d_Decomp -------------------------------*/
/* CUDA kernel for Decomp                                                  */
/*                                                                         */
/* @param[in]     ncells32 A multiple of 32 grid cells                     */
/* @param[in,out] A        Row-compressed matrix with zero fill            */
/*-------------------------------------------------------------------------*/
__global__
void d_Decomp(unsigned long const ncells32, float A[])
{
    /* Row element divided by diagonal */
    float piv;

    unsigned long idx = blockDim.x*blockIdx.x + threadIdx.x;
    if(idx < ncells32) {
        A += idx;
        piv = A[2*ncells32]/A[0];
        A[2*ncells32] = piv;
        A[4*ncells32] = A[4*ncells32] - A[ncells32]*piv;
        piv = A[6*ncells32]/A[0];
        A[6*ncells32] = piv;
        A[8*ncells32] = A[8*ncells32] - A[ncells32]*piv;
        piv = A[7*ncells32]/A[3*ncells32];
        A[7*ncells32] = piv;
        A[8*ncells32] = -A[4*ncells32]*piv + A[8*ncells32];
        A[10*ncells32] = A[10*ncells32] - A[5*ncells32]*piv;
        piv = A[11*ncells32]/A[3*ncells32];
        A[11*ncells32] = piv;
        A[12*ncells32] = A[12*ncells32] - A[4*ncells32]*piv;
        A[14*ncells32] = A[14*ncells32] - A[5*ncells32]*piv;
        piv = A[12*ncells32]/A[8*ncells32];
        A[12*ncells32] = piv;
        A[13*ncells32] = A[13*ncells32] - A[9*ncells32]*piv;
        A[14*ncells32] = -A[10*ncells32]*piv + A[14*ncells32];
        piv = A[15*ncells32]/A[3*ncells32];
        A[15*ncells32] = piv;
        A[16*ncells32] = A[16*ncells32] - A[4*ncells32]*piv;
        A[18*ncells32] = A[18*ncells32] - A[5*ncells32]*piv;
        piv = A[16*ncells32]/A[8*ncells32];
        A[16*ncells32] = piv;
        A[17*ncells32] = A[17*ncells32] - A[9*ncells32]*piv;
        A[18*ncells32] = -A[10*ncells32]*piv + A[18*ncells32];
        piv = A[17*ncells32]/A[13*ncells32];
        A[17*ncells32] = piv;
        A[18*ncells32] = -A[14*ncells32]*piv + A[18*ncells32];
    }
} /* END d_Decomp */



/*--------------------------------- Decomp --------------------------------*/
/* In-place sparse LU decomposition                                        */
/*                                                                         */
/* @param[in]     ncells32 A multiple of 32 grid cells                     */
/* @param[in,out] d_A      Row-compressed matrix with zero fill in device memory */
/*-------------------------------------------------------------------------*/
int Decomp(unsigned long const ncells32, float d_A[])
{

    unsigned long nBlocks = ((ncells32 + 127) & ~127) >> 7;
    unsigned long nThreads = 128;
    d_Decomp<<<nBlocks, nThreads>>>(ncells32, d_A);
    return 0;
} /* END Decomp */


/*------------------------- END small_decomp.h END ------------------------*/
