/*--------------------- BEGIN small_function.cu BEGIN ---------------------*/
/* @file small_function.cu                                                 */
/* @author jlinford                                                        */
/* @date 2012-06-22 15:44:04.020292                                        */
/* @brief The ODE function of the chemical model                           */
/*                                                                         */
/* The ODE function of the chemical model                                  */
/*                                                                         */
/* This file was generated by Kppa: http://www.paratools.com/Kppa          */
/*-------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <math.h>
#include "small_cu_parameters.h"
#include "small_sparse.h"
#include "small_function.h"

/*--------------------------------- d_Fun ---------------------------------*/
/* CUDA kernel for Fun                                                     */
/*                                                                         */
/* @param[in]     ncells32 A multiple of 32 grid cells                     */
/* @param[in]     var      Variable species concentrations                 */
/* @param[in]     fix      Fixed species concentrations                    */
/* @param[in]     rct      Reaction rates                                  */
/* @param[out]    vardot   The ODE function                                */
/*-------------------------------------------------------------------------*/
__global__
void d_Fun(unsigned long const ncells32, float var[], float fix[], float
        rct[], float vardot[])
{
    /* Reaction rates for the thread block */
    __shared__ float scratch[320];

    float * r = scratch + (threadIdx.x & 31);
    
    unsigned long widx = threadIdx.x >> 5;
    unsigned long idx = (blockIdx.x << 5) + (threadIdx.x & 31);
    
    var += idx;
    fix += idx;
    rct += idx;
    vardot += idx;
    
    switch(widx) {
    case 0:
        r[0] = fix[ncells32]*rct[0];
        r[32] = 8.018e-17*fix[ncells32]*var[ncells32];
        r[64] = rct[2*ncells32]*var[2*ncells32];
        break;
    case 1:
        r[96] = 1.576e-15*var[2*ncells32]*var[ncells32];
        r[128] = rct[4*ncells32]*var[2*ncells32];
        r[160] = 7.11e-11*fix[0]*var[0];
        break;
    case 2:
        r[192] = 1.2e-10*var[0]*var[2*ncells32];
        r[224] = 6.062e-15*var[2*ncells32]*var[3*ncells32];
        break;
    case 3:
        r[256] = 1.069e-11*var[4*ncells32]*var[ncells32];
        r[288] = rct[9*ncells32]*var[4*ncells32];
        break;
    }

    __syncthreads();
    
    switch(widx) {
    case 0:
        vardot[0] = r[128] - r[160] - r[192];
        vardot[ncells32] = 2.0*r[0] + r[160] - r[256] + r[288] - r[32] +
                r[64] - r[96];
        break;
    case 1:
        vardot[2*ncells32] = -r[128] - r[192] - r[224] + r[32] - r[64] -
                r[96];
        break;
    case 2:
        vardot[3*ncells32] = -r[224] + r[256] + r[288];
        break;
    case 3:
        vardot[4*ncells32] = r[224] - r[256] - r[288];
        break;
    }
} /* END d_Fun */



/*---------------------------------- Fun ----------------------------------*/
/* The ODE function of the chemical model                                  */
/*                                                                         */
/* @param[in]     ncells32 A multiple of 32 grid cells                     */
/* @param[in]     d_var    Variable species concentrations in device memory */
/* @param[in]     d_fix    Fixed species concentrations in device memory   */
/* @param[in]     d_rct    Reaction rates in device memory                 */
/* @param[out]    d_vardot The ODE function in device memory               */
/*-------------------------------------------------------------------------*/
void Fun(unsigned long const ncells32, float d_var[], float d_fix[], float
        d_rct[], float d_vardot[])
{
    unsigned long nBlocks = ncells32 >> 5;
    unsigned long nThreads = 128;
    d_Fun<<<nBlocks, nThreads>>>(ncells32, d_var, d_fix, d_rct, d_vardot);
} /* END Fun */



/*------------------------ END small_function.h END -----------------------*/
