/*--------------------- BEGIN small_jacobian.cu BEGIN ---------------------*/
/* @file small_jacobian.cu                                                 */
/* @author jlinford                                                        */
/* @date 2012-06-22 15:44:04.176826                                        */
/* @brief The Jacobian of the chemical model                               */
/*                                                                         */
/* The Jacobian of the chemical model                                      */
/*                                                                         */
/* This file was generated by Kppa: http://www.paratools.com/Kppa          */
/*-------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <math.h>
#include "small_cu_parameters.h"
#include "small_jacobian.h"

/*--------------------------------- d_Jac ---------------------------------*/
/* CUDA kernel for Jac                                                     */
/*                                                                         */
/* @param[in]     ncells32 A multiple of 32 grid cells                     */
/* @param[in]     var      Variable species concentrations                 */
/* @param[in]     fix      Fixed species concentrations                    */
/* @param[in]     rct      Reaction rates                                  */
/* @param[out]    sjac     The row-compressed sparse Jacobian              */
/*-------------------------------------------------------------------------*/
__global__
void d_Jac(unsigned long const ncells32, float var[], float fix[], float
        rct[], float sjac[])
{
    /* Scratch shared memory */
    __shared__ float scratch[416];

    float * t = scratch + (threadIdx.x & 31);
    
    unsigned long widx = threadIdx.x >> 5;
    unsigned long idx = (blockIdx.x << 5) + (threadIdx.x & 31);

    var += idx;
    fix += idx;
    rct += idx;
    sjac += idx;

    switch(widx) {
    case 0:
        t[0] = 7.11e-11*fix[0];
        t[32] = 1.2e-10*var[2*ncells32];
        t[64] = 8.018e-17*fix[ncells32];
        t[96] = 1.576e-15*var[2*ncells32];
        break;
    case 1:
        t[128] = 1.069e-11*var[4*ncells32];
        t[160] = rct[2*ncells32];
        t[192] = 1.576e-15*var[ncells32];
        break;
    case 2:
        t[224] = rct[4*ncells32];
        t[256] = 1.2e-10*var[0];
        t[288] = 6.062e-15*var[3*ncells32];
        break;
    case 3:
        t[320] = 6.062e-15*var[2*ncells32];
        t[352] = 1.069e-11*var[ncells32];
        t[384] = rct[9*ncells32];
        break;
    }

    __syncthreads();

    switch(widx) {
    case 0:
        sjac[0] = -t[0] - t[32];
        sjac[ncells32] = t[224] - t[256];
        sjac[2*ncells32] = t[0];
        sjac[3*ncells32] = -t[128] - t[64] - t[96];
        sjac[4*ncells32] = t[160] - t[192];
        break;
    case 1:
        sjac[5*ncells32] = -t[352] + t[384];
        sjac[6*ncells32] = -t[32];
        sjac[7*ncells32] = t[64] - t[96];
        sjac[8*ncells32] = -t[160] - t[192] - t[224] - t[256] - t[288];
        sjac[9*ncells32] = -t[320];
        break;
    case 2:
        sjac[10*ncells32] = 0;
        sjac[11*ncells32] = t[128];
        sjac[12*ncells32] = -t[288];
        sjac[13*ncells32] = -t[320];
        sjac[14*ncells32] = t[352] + t[384];
        break;
    case 3:
        sjac[15*ncells32] = -t[128];
        sjac[16*ncells32] = t[288];
        sjac[17*ncells32] = t[320];
        sjac[18*ncells32] = -t[352] - t[384];
        break;
    }
} /* END d_Jac */



/*---------------------------------- Jac ----------------------------------*/
/* The Jacobian of variable species                                        */
/*                                                                         */
/* @param[in]     ncells32 A multiple of 32 grid cells                     */
/* @param[in]     d_var    Variable species concentrations in device memory */
/* @param[in]     d_fix    Fixed species concentrations in device memory   */
/* @param[in]     d_rct    Reaction rates in device memory                 */
/* @param[out]    d_sjac   The row-compressed sparse Jacobian in device memory */
/*-------------------------------------------------------------------------*/
void Jac(unsigned long const ncells32, float d_var[], float d_fix[], float
        d_rct[], float d_sjac[])
{
    unsigned long nBlocks = ncells32 >> 5;
    unsigned long nThreads = 128;
    d_Jac<<<nBlocks, nThreads>>>(ncells32, d_var, d_fix, d_rct, d_sjac);
} /* END Jac */



/*------------------------ END small_jacobian.h END -----------------------*/
