!----------------------- BEGIN small_main.f90 BEGIN -----------------------
! @file small_main.f90                                                     
! @author jlinford                                                         
! @date 2012-06-22 15:44:04.278327                                         
! @brief Fortran90 driver with plplot                                      
!                                                                          
! Fortran90 driver with plplot                                             
!                                                                          
! This file was generated by Kppa: http://www.paratools.com/Kppa           
!--------------------------------------------------------------------------


PROGRAM main

    USE small_parameters
    USE small_integrate
    USE small_initialize
    USE small_driver_parameters

    IMPLICIT NONE

    !---------------------------------------------------------------------------
    ! Lookat data                                                               
    !---------------------------------------------------------------------------
    
    INTEGER, PARAMETER :: lookatFile = 100
    CHARACTER(80), PARAMETER :: fname = "small.dat"
    

    !---------------------------------------------------------------------------
    ! Program data                                                               
    !---------------------------------------------------------------------------
    
    ! Integration time 
    REAL(4) :: time

    ! Absolute integration tolerances for variable species 
    REAL(4) :: abstol(5)
    ! Relative integration tolerances for variable species 
    REAL(4) :: reltol(5)

    ! Integer integration in/out parameters 
    INTEGER :: idata(20)
    ! Real value integration in/out parameters 
    REAL(4) :: rdata(20)

    ! Species concentrations for all grid cells
    REAL(4), ALLOCATABLE :: conc(:,:)

    ! Last stepsize in each grid cell
    REAL(4), ALLOCATABLE :: lastH(:)
    
    ! Integrator statistics
    INTEGER :: nFun = 0       ! Number of function evaluations
    INTEGER :: nJac = 0       ! Number of Jacobian evaluations
    INTEGER :: nStp = 0       ! Number of solver steps
    INTEGER :: nAcc = 0       ! Number of accepted steps
    INTEGER :: nRej = 0       ! Number of rejected steps
    INTEGER :: nDec = 0       ! Number of matrix decompositions
    INTEGER :: nSol = 0       ! Number of Ax=b solves
    INTEGER :: nSng = 0       ! Number of singular decomposition results
    
    ! Iterator
    INTEGER :: i
   
    !---------------------------------------------------------------------------
    ! Program text                                                               
    !---------------------------------------------------------------------------

    WRITE(*,101) NCELLS,TSTART,TEND
101 FORMAT("Kppa: Integrating ",I8," grid cells, time interval [",E8.2,",",E8.2,"]")

    ! Initialize example grid data
    ALLOCATE(conc(NSPEC,NCELLS))
    DO i=1,ncells
        CALL Initialize(conc(1:NVAR,i), conc(NVAR+1:NSPEC,i))
    END DO
    
    ! Allocate storage for last step size (useful in reentrant solvers)
    ALLOCATE(lastH(ncells))

    ! Set time interval
    time = TSTART

    ! Initialize vector tolerances
    abstol = ATOLS
    reltol = RTOLS

    ! Initialize integration parameters to 0 = use defaults
    idata = 0
    rdata = 0

    ! Rosenbrock default parameters
    idata(1) = 0       ! System is non-autonomous: F = F(t,y)
    idata(2) = 0       ! Use vector tolerances
    idata(3) = 100000  ! Maximum number of integration steps
    idata(4) = 5       ! Rodas4 Rosenbrock method
    idata(5) = 0       ! Tolerance vectors will not be checked

    rdata(1) = 0       ! Integration step size lower bound: 0 recommended
    rdata(2) = 0       ! Integration step size upper bound: 0 recommended
    rdata(3) = TDEL ! Starting integration step size
    rdata(4) = 0.2     ! Lower bound on step decrease factor
    rdata(5) = 6       ! Upper bound on step increase factor
    rdata(6) = 0.1     ! Step decrease factor after step rejection
    rdata(7) = 0.9     ! Safety factor in the computation of new step size

    ! Initialize LookAt file
    OPEN(unit=lookatFile,file=fname)

    ! Initialize species monitor
    CALL InitMonitor(rdata(3))


    ! Perform time integration
    DO WHILE (time < TEND)
        CALL GridIntegrate(NCELLS, conc, time, time+TDEL, &
                           abstol, reltol, idata, rdata, lastH)

        ! Advance time
        time = time + TDEL

        ! Accumulate statistics
        nStp = nStp + idata(13)
        nAcc = nAcc + idata(14)
        nRej = nRej + idata(15)
        nSng = nSng + idata(18) 
        nFun = nFun + idata(11)
        nJac = nJac + idata(12)
        nDec = nDec + idata(16)
        nSol = nSol + idata(17)

        ! Write species concentrations to file
        CALL LookAt(rdata(12), rdata(13))

        
        ! Report species concentrations to stdout
        CALL Monitor(rdata(12), rdata(13))

        
        ! Use last integration step size as the starting step size
        rdata(3) = rdata(12)
    END DO

    ! Report statistics                       
    WRITE(*,102) nStp,nAcc,nRej,nSng,nFun,nJac,nDec,nSol
102 FORMAT("Kppa: Stp=",I12,", Acc=",I12,", Rej=",I12,", Sng=",I12, &
               ", Fun=",I12,", Jac=",I12,", Dec=",I12,", Sol=",I12)

    ! Clean up and exit
    CLOSE(lookatFile)
    DEALLOCATE(conc)
    DEALLOCATE(lastH)
    STOP


    CONTAINS

!-------------------------------- Monitor ---------------------------------
! Reports on species concentrations as they are calculated                 
!                                                                          
! @param[in]     step Last accepted integrator time step (H)               
! @param[in]     err  Error in solution vector                             
!--------------------------------------------------------------------------
    SUBROUTINE Monitor(step, err)
        IMPLICIT NONE

        REAL(4), INTENT(IN) :: step
        REAL(4), INTENT(IN) :: err

        REAL :: perc
        REAL(4) :: cspc(NMONITOR)

        cspc(1) = conc(1,1) / CFACTOR
        cspc(2) = conc(2,1) / CFACTOR
        cspc(3) = conc(3,1) / CFACTOR
        cspc(4) = conc(4,1) / CFACTOR
        cspc(5) = conc(5,1) / CFACTOR
        cspc(6) = conc(7,1) / CFACTOR

        perc = ((time-TSTART)/abs(TEND-TSTART))*100.0       
        WRITE(*,301) perc,time,step,err,cspc
301     FORMAT(F5.1,"% | ",F9.2," | ",E9.3," | ",E9.3," | ",6(E11.4," |"))
    END SUBROUTINE Monitor



!------------------------------ InitMonitor -------------------------------
! Initializes the species monitor                                          
!                                                                          
! @param[in]     step Last accepted integrator time step (H)               
!--------------------------------------------------------------------------
    SUBROUTINE InitMonitor(step)
        IMPLICIT NONE

        REAL(4), INTENT(IN) :: step


        WRITE(*,401)
        WRITE(*,402) "O1D","O","O3","NO","NO2","O2"
        WRITE(*,401)
401     FORMAT(123("-"))
402     FORMAT("       |     T     |     H     | Err. Norm | ",30(A11," |"))


        CALL Monitor(step, 0.0)

    END SUBROUTINE InitMonitor



!--------------------------------- LookAt ---------------------------------
! Writes species concentrations to file                                    
!                                                                          
! @param[in]     step Last accepted integrator time step (H)               
! @param[in]     err  Error in solution vector                             
!--------------------------------------------------------------------------
    SUBROUTINE LookAt(step, err)
        IMPLICIT NONE

        REAL(4), INTENT(IN) :: step
        REAL(4), INTENT(IN) :: err

    REAL(4) :: data(3 + NLOOKAT)

    data(1) = time
    data(2) = step
    data(3) = err
    data(1) = conc(3, 1) / CFACTOR
    data(2) = conc(5, 1) / CFACTOR
    WRITE(lookatFile,*) data
    END SUBROUTINE LookAt



END PROGRAM main


!------------------------- END small_main.f90 END -------------------------
