/*----------------------- BEGIN small_solve.cu BEGIN ----------------------*/
/* @file small_solve.cu                                                    */
/* @author jlinford                                                        */
/* @date 2012-06-22 15:44:04.114365                                        */
/* @brief Solve Ax=b by forward-backward substitution                      */
/*                                                                         */
/* Solve Ax=b by forward-backward substitution                             */
/*                                                                         */
/* This file was generated by Kppa: http://www.paratools.com/Kppa          */
/*-------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <math.h>
#include "small_cu_parameters.h"
#include "small_sparse.h"
#include "small_solve.h"

/*-------------------------------- d_Solve --------------------------------*/
/* CUDA kernel for Solve                                                   */
/*                                                                         */
/* @param[in]     ncells32 A multiple of 32 grid cells                     */
/* @param[in]     A        Row-compressed matrix with zero fill            */
/* @param[in,out] x        Input and solution vector                       */
/*-------------------------------------------------------------------------*/
__global__
void d_Solve(unsigned long const ncells32, float A[], float x[])
{

    unsigned long tidx = blockDim.x*blockIdx.x + threadIdx.x;
    if(tidx < ncells32) {
        float x0;
        float x1;
        float x2;
        float x3;
        float x4;

        A += tidx;
        x += tidx;
        x0 = x[0];
        x1 = -A[2*ncells32]*x0 + x[ncells32];
        x2 = -A[6*ncells32]*x0 - A[7*ncells32]*x1 + x[2*ncells32];
        x3 = -A[11*ncells32]*x1 - A[12*ncells32]*x2 + x[3*ncells32];
        x4 = -A[15*ncells32]*x1 - A[16*ncells32]*x2 - A[17*ncells32]*x3 +
                x[4*ncells32];

        x4 = x4/A[18*ncells32];
        x3 = (-A[14*ncells32]*x4 + x3)/A[13*ncells32];
        x2 = (-A[10*ncells32]*x4 - A[9*ncells32]*x3 + x2)/A[8*ncells32];
        x1 = (-A[4*ncells32]*x2 - A[5*ncells32]*x4 + x1)/A[3*ncells32];
        x0 = (-A[ncells32]*x2 + x0)/A[0];
        x[4*ncells32] = x4;
        x[3*ncells32] = x3;
        x[2*ncells32] = x2;
        x[ncells32] = x1;
        x[0] = x0;
    }
} /* END d_Solve */



/*--------------------------------- Solve ---------------------------------*/
/* Solve Ax=b in place by back substitution                                */
/*                                                                         */
/* @param[in]     ncells32 A multiple of 32 grid cells                     */
/* @param[in]     d_A      Row-compressed matrix with zero fill in device memory */
/* @param[in,out] d_x      Input and solution vector in device memory      */
/*-------------------------------------------------------------------------*/
void Solve(unsigned long const ncells32, float d_A[], float d_x[])
{

    unsigned long nBlocks = ((ncells32 + 127) & ~127) >> 7;
    unsigned long nThreads = 128;
    d_Solve<<<nBlocks, nThreads>>>(ncells32, d_A, d_x);
} /* END Solve */



/*------------------------- END small_solve.h END -------------------------*/
