/*----------------------- BEGIN small_sparse.h BEGIN ----------------------*/
/* @file small_sparse.h                                                    */
/* @author jlinford                                                        */
/* @date 2012-06-22 15:44:04.018888                                        */
/* @brief Data and utilities for row-compressed sparse matrices            */
/*                                                                         */
/* The following matrices are represented here in row-compressed form:     */
/* @li The Jacobian                                                        */
/* @li The LU decomposition of the Jacobian                                */
/* @li The stoichiometric matrix                                           */
/*                                                                         */
/* This file was generated by Kppa: http://www.paratools.com/Kppa          */
/*-------------------------------------------------------------------------*/

#ifndef __SMALL_SPARSE_H__
#define __SMALL_SPARSE_H__

#ifdef __cplusplus
extern "C" {
#endif



/* Row indices of elements in the row-compressed Jacobian */
__constant__
int const JAC_IROW[19] = { 0, 0, 1, 1, 1, 1, 2, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4,
        4, 4 };


/* Column indices of elements in the row-compressed Jacobian */
__constant__
int const JAC_ICOL[19] = { 0, 2, 0, 1, 2, 4, 0, 1, 2, 3, 4, 1, 2, 3, 4, 1, 2,
        3, 4 };


/* Start-of-row indices in the row-compressed Jacobian */
__constant__
int const JAC_CROW[6] = { 0, 2, 6, 11, 15, 19 };


/* Diagonal indices in the row-compressed Jacobian */
__constant__
int const JAC_DIAG[6] = { 0, 3, 8, 13, 18, 19 };


/* Row indices of elements in the row-compressed LU decomposition of the Jacobian */
__constant__
int const JAC_LU_IROW[19] = { 0, 0, 1, 1, 1, 1, 2, 2, 2, 2, 2, 3, 3, 3, 3, 4,
        4, 4, 4 };


/* Column indices of elements in the row-compressed LU decomposition of the Jacobian */
__constant__
int const JAC_LU_ICOL[19] = { 0, 2, 0, 1, 2, 4, 0, 1, 2, 3, 4, 1, 2, 3, 4, 1,
        2, 3, 4 };


/* Start-of-row indices in the row-compressed LU decomposition of the Jacobian */
__constant__
int const JAC_LU_CROW[6] = { 0, 2, 6, 11, 15, 19 };


/* Diagonal indices in the row-compressed LU decomposition of the Jacobian */
__constant__
int const JAC_LU_DIAG[6] = { 0, 3, 8, 13, 18, 19 };


/* Row indices of elements in the row-compressed stoichiometric matrix */
__constant__
int const STOICH_IROW[34] = { 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2,
        3, 3, 3, 4, 4, 4, 6, 6, 6, 6, 6, 6, 6, 6, 7, 7, 7, 7 };


/* Column indices of elements in the row-compressed stoichiometric matrix */
__constant__
int const STOICH_ICOL[34] = { 4, 5, 6, 0, 1, 2, 3, 5, 8, 9, 1, 2, 3, 4, 6, 7,
        7, 8, 9, 7, 8, 9, 0, 1, 2, 3, 4, 6, 7, 8, 0, 2, 4, 9 };


/* Start-of-row indices in the row-compressed stoichiometric matrix */
__constant__
int const STOICH_CROW[10] = { 0, 3, 10, 16, 19, 22, 22, 30, 34, 34 };


/* Stoichiometric coefficients */
__constant__
float const STOICH[34] = { 1.0, -1.0, -1.0, 2.0, -1.0, 1.0, -1.0, 1.0, -1.0,
        1.0, 1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, 1.0, 1.0, 1.0, -1.0,
        -1.0, -1.0, -1.0, 1.0, 2.0, 1.0, 2.0, 1.0, 1.0, -1.0, -1.0, -1.0,
        -1.0 };





#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* __SMALL_SPARSE_H__ */
/*------------------------- END small_sparse.h END ------------------------*/
